@@ -8,7 +8,17 @@
 # under certain conditions; see COPYING for details.
 
 # Debian releases
-RELEASE_etch="4.0 r4"
-RELEASE_lenny="5.0 r0"
+RELEASE_etch="4.0 r8"
+RELEASE_lenny="5.0.1"
 RELEASE_squeeze="testing"
 RELEASE_sid="unstable"
+
+# Ubuntu releases
+RELEASE_dapper="6.06"
+RELEASE_edgy="6.10"
+RELEASE_feisty="7.04"
+RELEASE_gutsy="7.10"
+RELEASE_hardy="8.04"
+RELEASE_intrepid="8.10"
+RELEASE_jaunty="9.04"
+RELEASE_karmic="9.10"
